#!/bin/bash
echored () {
	echo "${TEXTRED}$1${TEXTRESET}"
}
echogreen () {
	echo "${TEXTGREEN}$1${TEXTRESET}"
}
build_zlib() {
  export ZPREFIX=$BUILD/zlib/$LARCH
  rm -rf $ZPREFIX 2>/dev/null
	echogreen "Building ZLib..."
	[ -f "zlib-$ZVER.tar.gz" ] || wget http://zlib.net/zlib-$ZVER.tar.gz
	[ -d zlib-$ZVER ] || tar -xf zlib-$ZVER.tar.gz
	cd zlib-$ZVER
	./configure --prefix=$ZPREFIX
	[ $? -eq 0 ] || { echored "Zlib configure failed!"; exit 1; }
	make -j$JOBS
	[ $? -eq 0 ] || { echored "Zlib build failed!"; exit 1; }
	make install
  make clean
	cd $DIR
}
build_bzip2() {
  export BPREFIX=$BUILD/bzip2/$LARCH
  rm -rf $BPREFIX 2>/dev/null
	echogreen "Building BZip2..."
	[ -f "bzip2-latest.tar.gz" ] || wget https://www.sourceware.org/pub/bzip2/bzip2-latest.tar.gz
	[[ -d "bzip2-"[0-9]* ]] || tar -xf bzip2-latest.tar.gz
	cd bzip2-[0-9]*
	sed -i -e '/# To assist in cross-compiling/,/LDFLAGS=/d' -e "s/CFLAGS=/CFLAGS=$CFLAGS /" -e 's/bzip2recover test/bzip2recover/' Makefile
	export LDFLAGS
	make -j$JOBS
	export -n LDFLAGS
	[ $? -eq 0 ] || { echored "Bzip2 build failed!"; exit 1; }
	make install -j$JOBS PREFIX=$BPREFIX
  make clean
	cd $DIR
}
build_pcre() {
	build_zlib
	build_bzip2
  export PPREFIX=$BUILD/pcre/$LARCH
  rm -rf $PPREFIX 2>/dev/null
	echogreen "Building PCRE..."
	[ -f "pcre-$PVER.tar.bz2" ] || wget https://ftp.pcre.org/pub/pcre/pcre-$PVER.tar.bz2
	[ -d pcre-$PVER ] || tar -xf pcre-$PVER.tar.bz2
	cd pcre-$PVER
	$STATIC && local FLAGS="--disable-shared "
  ./configure $FLAGS--prefix= \
              --enable-unicode-properties \
              --enable-jit \
              --enable-pcregrep-libz \
              --enable-pcregrep-libbz2 \
              --host=$TARGET_HOST \
              --target=$TARGET_HOST \
              CFLAGS="$CFLAGS -I$ZPREFIX/include -I$BPREFIX/include" \
              LDFLAGS="$LDFLAGS -L$ZPREFIX/lib -L$BPREFIX/lib"
	[ $? -eq 0 ] || { echored "PCRE configure failed!"; exit 1; }
	make -j$JOBS
	[ $? -eq 0 ] || { echored "PCRE build failed!"; exit 1; }
	make install -j$JOBS DESTDIR=$PPREFIX
  make clean
	cd $DIR
  $STATIC || install -D $PPREFIX/lib/libpcre.so $DEST/$LARCH/lib/libpcre.so
}
build_gdbm() {
  export GPREFIX=$BUILD/gdbm/$LARCH
  rm -rf $GPREFIX 2>/dev/null
	echogreen "Building Gdbm..."
	[ -f "gdbm-latest.tar.gz" ] || wget http://mirrors.kernel.org/gnu/gdbm/gdbm-latest.tar.gz
	[[ -d "gdbm-"[0-9]* ]] || tar -xf gdbm-latest.tar.gz
	cd gdbm-[0-9]*
	$STATIC && local FLAGS="--disable-shared "
	./configure $FLAGS--prefix= \
              --disable-nls \
              --host=$TARGET_HOST \
              --target=$TARGET_HOST \
              CFLAGS="$CFLAG" \
              LDFLAGS="$LDFLAGS"
	[ $? -eq 0 ] || { echored "Gdbm configure failed!"; exit 1; }
	make -j$JOBS
	[ $? -eq 0 ] || { echored "Gdbm build failed!"; exit 1; }
	make install -j$JOBS DESTDIR=$GPREFIX
  make clean
	cd $DIR
  $STATIC || install -D $GPREFIX/lib/libgdbm.so $DEST/$LARCH/lib/libgdbm.so.6
}
build_ncurses() {
  export NPREFIX=$BUILD/ncurses/$LARCH
  rm -rf $NPREFIX 2>/dev/null
	echogreen "Building NCurses..."
	[ -f "ncurses-$NVER.tar.gz" ] || wget http://mirrors.kernel.org/gnu/ncurses/ncurses-$NVER.tar.gz
	[ -d "ncurses-$NVER" ] || tar -xf ncurses-$NVER.tar.gz
	cd ncurses-$NVER
	$STATIC && local FLAGS="--disable-shared "
	./configure $FLAGS--prefix=$NPREFIX \
              --disable-nls \
              --disable-stripping \
              --host=$TARGET_HOST \
              --target=$TARGET_HOST \
              CFLAGS="$CFLAGS" \
              LDFLAGS="$LDFLAGS"
  [ $? -eq 0 ] || { echored "Ncurses configure failed!"; exit 1; }
	make -j$JOBS
	[ $? -eq 0 ] || { echored "NCurses build failed!"; exit 1; }
	make install -j$JOBS
  make clean
	cd $DIR
}
build_zsh() {
  PREFIX=$BUILD/zsh/$LARCH
  rm -rf $PREFIX 2>/dev/null
  echogreen "Building Zsh..."
  [ -f "zsh-$VER.tar.xz" ] || wget -O zsh-$VER.tar.xz https://sourceforge.net/projects/zsh/files/zsh/$VER/zsh-$VER.tar.xz/download
	[ -d "zsh-$VER" ] || tar -xf zsh-$VER.tar.gz
	cd zsh-$VER
  sed -i "/exit 0/d" Util/preconfig
  . Util/preconfig
	$STATIC && local FLAGS="--disable-shared --disable-dynamic --disable-dynamic-nss "
  ./configure \
    --host=$TARGET_HOST \
    --target=$TARGET_HOST \
    --enable-cflags="$CFLAGS -I $PPREFIX/include -I$GPREFIX/include -I$NPREFIX/include" \
    --enable-ldflags="$LDFLAGS -L$PPREFIX/lib -L$GPREFIX/lib -L$NPREFIX/lib" \
    $FLAGS--prefix=/system \
		--bindir=/system/bin \
		--sbindir=/sbin \
    --libexecdir=/system/bin \
		--sysconfdir=/system/etc \
    --enable-etcdir=/system/etc \
		--datarootdir=/system/usr/share \
		--enable-zshenv=/system/etc/zsh/zshenv \
		--enable-zprofile=/system/etc/zsh/zprofile \
		--enable-zlogin=/system/etc/zsh/zlogin \
		--enable-zlogout=/system/etc/zsh/zlogout \
		--enable-site-fndir=/system/usr/share/zsh/functions \
		--enable-fndir=/system/usr/share/zsh/functions \
		--enable-scriptdir=/system/usr/share/zsh/scripts \
		--enable-site-scriptdir=/system/usr/share/zsh/scripts \
    --enable-runhelpdir=/system/usr/share/zsh/runhelp \
		--enable-function-subdirs \
    --enable-multibyte \
		--enable-pcre \
    --disable-restricted-r
  [ $? -eq 0 ] || { echored "Zsh configure failed!"; exit 1; }
	make -j$JOBS
	[ $? -eq 0 ] || { echored "Zsh build failed!"; exit 1; }
	make install -j$JOBS DESTDIR=$PREFIX
  make clean
	cd $DIR
  install -D $BUILD/zsh/$LARCH/system/bin/zsh $DEST/$LARCH/bin/zsh
  $STATIC || { mkdir -p $DEST/$LARCH/usr/share; cp -rf $PREFIX/system/usr/share/zsh $DEST/$LARCH/usr/share/; }
}
setup_ohmyzsh() {
  local OPREFIX=$BUILD/ohmyzsh/$LARCH
  rm -rf $OPREFIX 2>/dev/null
  mkdir -p $OPREFIX
  git clone https://github.com/ohmyzsh/ohmyzsh.git $OPREFIX/.oh-my-zsh
  cd $OPREFIX
  cp $OPREFIX/.oh-my-zsh/templates/zshrc.zsh-template .zshrc
  sed -i -e "s|PATH=.*|PATH=\$PATH|" -e "s|ZSH=.*|ZSH=/system/etc/zsh/.oh-my-zsh|" -e "s|ARCHFLAGS=.*|ARCHFLAGS=\"-arch $LARCH\"|" .zshrc
  cd $DIR
  mkdir -p $DEST/$LARCH/etc/zsh
  cp -rf $OPREFIX/.oh-my-zsh $DEST/$LARCH/etc/zsh/
  cp -f $OPREFIX/.zshrc $DEST/$LARCH/etc/zsh/.zshrc
}

NDK=r20b
export ANDROID_NDK_HOME=$PWD/android-ndk-$NDK
export HOST_TAG=linux-x86_64
export MIN_SDK_VERSION=21
export STATIC=true
export ARCH="arm arm64 x86 x64"


TEXTRESET=$(tput sgr0)
TEXTGREEN=$(tput setaf 2)
TEXTRED=$(tput setaf 1)
DIR=$PWD
if [ -f /proc/cpuinfo ]; then
  export JOBS=$(grep flags /proc/cpuinfo | wc -l)
elif [ ! -z $(which sysctl) ]; then
  export JOBS=$(sysctl -n hw.ncpu)
else
  export JOBS=2
fi

# Set up Android NDK
[ -f "android-ndk-$NDK-$HOST_TAG.zip" ] || wget https://dl.google.com/android/repository/android-ndk-$NDK-$HOST_TAG.zip
[ -d "android-ndk-$NDK" ] || unzip -qo android-ndk-$NDK-$HOST_TAG.zip
export TOOLCHAIN=$ANDROID_NDK_HOME/toolchains/llvm/prebuilt/$HOST_TAG
export PATH=$TOOLCHAIN/bin:$PATH
$STATIC && { CFLAGS="--static "; LDFLAGS="--static "; DEST=$DIR/zsh-static; } || DEST=$DIR/zsh-dynamic
export CFLAGS="$CFLAGS-O3"
export LDFLAGS="$LDFLAGS-s"
export VER=5.7.1
export NVER=6.1
export PVER=8.43
export ZVER=1.2.11

for i in $ARCH; do
  case $i in
    "arm64"|"aarch64") export TARGET_HOST=aarch64-linux-android; LARCH=arm64-v8a;;
    "arm"|"armeabi"|"armeabi_v7a") export TARGET_HOST=armv7a-linux-androideabi; LARCH=armeabi-v7a;;
    "x86"|"i686") export TARGET_HOST=i686-linux-android; LARCH=x86;;
    "x64"|"x86_64") export TARGET_HOST=x86_64-linux-android; LARCH=x86_64;;
    *) echo "Invalid ARCH: $i!"; exit 1;;
  esac

  [ "$LARCH" == "armeabi-v7a" ] && export TARGET_HOST=arm-linux-androideabi
  export AR=$TOOLCHAIN/bin/$TARGET_HOST-ar
  export AS=$TOOLCHAIN/bin/$TARGET_HOST-as
  export LD=$TOOLCHAIN/bin/$TARGET_HOST-ld
  export RANLIB=$TOOLCHAIN/bin/$TARGET_HOST-ranlib
  export STRIP=$TOOLCHAIN/bin/$TARGET_HOST-strip
  [ "$LARCH" == "armeabi-v7a" ] && export TARGET_HOST=armv7a-linux-androideabi
  export CC=$TOOLCHAIN/bin/$TARGET_HOST$MIN_SDK_VERSION-clang
  export CXX=$TOOLCHAIN/bin/$TARGET_HOST$MIN_SDK_VERSION-clang++
  $STATIC && export BUILD=$DIR/build-static || export BUILD=$DIR/build-dynamic
  
  rm -rf $DEST/$LARCH 2>/dev/null; mkdir -p $DEST/$LARCH
  build_pcre
  build_gdbm
  build_ncurses
  build_zsh
  setup_ohmyzsh
  ! $STATIC && [ "$LARCH" == "arm64-v8a" -o "$LARCH" == "x86_64" ] && mv -f $DEST/$LARCH/lib $DEST/$LARCH/lib64
done

echogreen "Output can be found in $DEST"
